﻿namespace AsyncSample.ViewModels
{
    using System.Threading;
    using System.Threading.Tasks;

    public class MainViewModel : NotificationObject
    {
        #region 公開プロパティ
        private string result = "まだ実行してないよ。";
        /// <summary>
        /// 実行結果を取得または設定します。
        /// </summary>
        public string Result
        {
            get { return result; }
            set { SetProperty(ref result, value); }
        }

        private bool isBusy;
        /// <summary>
        /// ビジー状態かどうかを取得または設定します。
        /// </summary>
        public bool IsBusy
        {
            get { return isBusy; }
            set
            {
                if (SetProperty(ref isBusy, value))
                    ButtonCommand.RaiseCanExecuteChanged();
            }
        }

        private DelegateCommand buttonCommand;
        /// <summary>
        /// ボタンコマンドを取得します。
        /// </summary>
        public DelegateCommand ButtonCommand
        {
            get
            {
                return buttonCommand ?? (buttonCommand = new DelegateCommand(
                    _ =>
                    {
                        System.Console.WriteLine("Thread[{0}] 非同期処理をコールします。", Thread.CurrentThread.ManagedThreadId);
                        HeavyWorkAsync();
                        System.Console.WriteLine("Thread[{0}] 非同期処理をコールしました。", Thread.CurrentThread.ManagedThreadId);
                    },
                    _ =>
                    {
                        return !IsBusy;
                    }));
            }
        }
        #endregion 公開プロパティ

        /// <summary>
        /// 重たい処理を非同期で実行します。
        /// </summary>
        private async void HeavyWorkAsync()
        {
            IsBusy = true;
            Result = "只今実行中...";
            System.Console.WriteLine("Thread[{0}] 非同期処理を実行します。", Thread.CurrentThread.ManagedThreadId);
            await HeavyWork();
            // ここで一旦 return される

            // 非同期処理が終了したらここから再開する
            System.Console.WriteLine("Thread[{0}] 非同期処理を実行しました。", Thread.CurrentThread.ManagedThreadId);
            Result = "終了しました。";
            IsBusy = false;
        }

        /// <summary>
        /// 重たい処理を実行するタスクを返します。
        /// </summary>
        /// <returns>重たい処理を実行するタスク</returns>
        private Task HeavyWork()
        {
            return Task.Run(() =>
            {
                System.Console.WriteLine("Thread[{0}] 重たい処理を実行します。", Thread.CurrentThread.ManagedThreadId);
                Thread.Sleep(3000);
                System.Console.WriteLine("Thread[{0}] 重たい処理を終了します。", Thread.CurrentThread.ManagedThreadId);
            });
        }
    }
}
